----------------------------------------------------------------------------------
-- Company: HeikosHomeCompany
-- Engineer: Heiko Lechner
-- 
-- Create Date:    17:11:47 01/12/2008 
-- Design Name: 
-- Module Name:    debouncer - Behavioral 
-- Project Name: 
-- Target Devices: Spartan3E
-- Tool versions: ISE 9.2.04i
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--Die IOs sind hier vom Digilent Spartan-3E FPGA Starter Kit Manual bernommen
entity DEBOUNCER_TEST is
	port	(
				CLK_50MHZ			: in  std_logic;
				BTN_NORTH			: in  bit;
				BTN_SOUTH			: in bit;
				LED0					: out std_logic;
				LED1					: out std_logic;
				LED2					: out std_logic;
				LED3					: out std_logic;
				LED7					: out  bit
			);
end DEBOUNCER_TEST;

architecture Behavioral of DEBOUNCER_TEST is


signal	LED_FIRST							: bit;
signal	LED_CE								: bit;
signal	CE										: bit;
signal	LED_COUNT							: std_logic_vector(3 downto 0);
signal	OUT_TMP								: bit;
signal	RESET									: bit;

--Einbindung der Prescaler- Komponente
component PRESCALER is
	generic	(
					--Bitbreite des Prescalers
					BIT_WIDTH	: positive
				);
	port		(
					--Systemtakt
					CLK				: in  std_logic;
					--Synchroner Reseteingang (active high)
					RESET				: in  bit;
					--Ausgang
					PRESCALER_OUT	: out  bit
				);
end component;

--Einbindung eines Zhlers (zhlt bei Tastendruck hoch)
--so kann man Tastenprellen sichtbar machen
component COUNTER_RST_CE is
	generic	(
					--Bitbreite des Zhlers
					BIT_WIDTH	: positive
				);
	port		(
					--Systemtakt
					CLK		: in  std_logic;
					--Synchroner Reseteingang (active high)
					RESET		: in  bit;
					--Clock enable (um den Zhler unter dem Systemtakt zu betreiben)
					CE			: in  bit;
					--Ausgang
					OUTPUT	: out std_logic_vector((BIT_WIDTH - 1) downto 0)
				);
end component;

--Einbindung des Entprellers als Komponente
component DEBOUNCER is
	generic	(
					--Bitbreite des Sample- Zhlers
					SAMPLE_BIT_WIDTH	:	positive
				);
	port		(
					--Systemtakt
					CLK					: in	std_logic;
					--Synchroner Reseteingang (active high)
					RESET					: in	bit;
					--Clock enable (hier kann der Systemtakt auf den Sample- Takt verringert werden)
					CE						: in	bit;
					--Eingang fr den prellenden Taster/Schalter
					BOUNCING_INPUT		: in	bit;
					--hoffentlich prellfreier Ausgang
					DEBOUNCED_OUTPUT	: out	bit
				);
end component;

for I_0: COUNTER_RST_CE use entity WORK.COUNTER_RST_CE(UP_COUNTER);
for I_1: DEBOUNCER use entity WORK.DEBOUNCER(Behavioral);
for I_2: PRESCALER use entity WORK.PRESCALER(Behavioral);

begin
	
	--Die Resetleitung wird auf den unteren Taster gelegt
	RESET <= BTN_SOUTH;
	
	--Der Prescaler sollte so den Systemtakt durch 2^15 teilen (->CE)
	I_2: PRESCALER
					generic map	(
											BIT_WIDTH	=> 15
									)
					port map		(
											CLK => CLK_50MHZ,
											RESET => RESET,
											PRESCALER_OUT => CE
									);
									
	--Der Entpreller entscheidet sich nach 2^4 aufeinanderfolgenden, gleichen 
	--abgetasteten Spannungspegeln fr einen prellfreien Zustand.
	--Prellender Eingang ist hier der obere Taster.
	--Abtastfrequenz wird durch den Prescaler gestellt
	I_1: DEBOUNCER
					generic map	(
											SAMPLE_BIT_WIDTH	=> 4
									)
					port map		(
											CLK => CLK_50MHZ,
											RESET => RESET,
											CE => CE,
											BOUNCING_INPUT => BTN_NORTH,
											DEBOUNCED_OUTPUT => OUT_TMP
									);
	--Der Zhler zhlt bei Tastendruck einen weiter (->LED_CE)
	I_0: COUNTER_RST_CE
						generic map
										(
											BIT_WIDTH => 4
										)
						port map		(
											CLK => CLK_50MHZ,
											RESET => RESET,
											CE => LED_CE,
											OUTPUT => LED_COUNT
										);
	
	--Die LEDs werden an den Zhler angeschlossen
	LED0 <= LED_COUNT(0);
	LED1 <= LED_COUNT(1);
	LED2 <= LED_COUNT(2);
	LED3 <= LED_COUNT(3);
	--LED7 leuchtet, wenn der Taster gedrckt wird
	LED7 <= OUT_TMP;
	
	--Dieser Prozess erzeugt einen LED_CE- Impuls, wenn der Taster gedrckt wurde
	LED_CE_GEN: process (CLK_50MHZ) 
	begin
		if rising_edge(CLK_50MHZ) then
			if RESET = '1' then
				LED_CE <= '0';
				LED_FIRST <= '1';
			--Ist der Taster gedrckt und LED_FIRST auf high...
			elsif (OUT_TMP = '1') and (LED_FIRST = '1') then
				--...dann wird LED_CE auf high gesetzt und LED_FIRST auf low,
				--so dass nur einen Takt lang LED_CE = high ist
				LED_FIRST <= '0';
				LED_CE <= '1';
			--Ist der Taster gedrckt und LED_FIRST nicht auf low...
			elsif (OUT_TMP = '1') then
				--...dann wird LED_CE abgeschaltet, weil es schon einen Takt aktiv war 
				LED_CE <= '0';
			--Ist der Taster nicht gedrckt...
			else
				--...dann kann LED_FIRST wieder auf high gesetzt werden (Schaltung wieder scharf)
				--LED_CE natrlich weiterhin low
				LED_CE <= '0';
				LED_FIRST <= '1';
			end if;
		end if;
	end process LED_CE_GEN;
	

														
end Behavioral;